# No shebang line, this module is meant to be imported
#
# Copyright 2013 Oliver Palmer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Logger
======

Contains the root logger for PyFarm as well as the default
message formatting.
"""

import os
import sys
import json
import logging
import warnings
from logging import Formatter

from pyfarm.core.enums import INTERACTIVE_INTERPRETER

# Import or construct the necessary objects depending on the Python version
# and use sys.version_info directly to avoid possible circular import issues.
PY_MAJOR, PY_MINOR = sys.version_info[0:2]
PY26 = PY_MAJOR, PY_MINOR == (2, 6)
if (PY_MAJOR, PY_MINOR) >= (2, 7):
    from logging import NullHandler, captureWarnings
    from logging.config import dictConfig
else:  # pragma: no cover
    from logutils import NullHandler
    from logutils.dictconfig import dictConfig
    _warnings_showwarning = None

    def _showwarning(message, category, filename, lineno, file=None, line=None):
        """
        .. note::
            This function is a copy of Python 2.7's ``_showwarning``

        Implementation of showwarnings which redirects to logging, which will
        first check to see if the file parameter is None. If a file is
        specified, it will delegate to the original warnings implementation of
        showwarning. Otherwise, it will call warnings.formatwarning and will
        log the resulting string to a warnings logger named "py.warnings" with
        level logging.WARNING.
        """
        if file is not None:
            if _warnings_showwarning is not None:
                _warnings_showwarning(
                    message, category, filename, lineno, file, line)
        else:
            s = warnings.formatwarning(
                message, category, filename, lineno, line)
            logger = logging.getLogger("py.warnings")
            if not logger.handlers:
                logger.addHandler(NullHandler())
            logger.warning("%s", s)

    def captureWarnings(capture):
        """
        .. note::
            This function is a copy of Python 2.7's ``captureWarnings``

        If capture is true, redirect all warnings to the logging package.
        If capture is False, ensure that warnings are not redirected to logging
        but to their original destinations.
        """
        global _warnings_showwarning
        if capture:
            if _warnings_showwarning is None:
                _warnings_showwarning = warnings.showwarning
                warnings.showwarning = _showwarning
        else:
            if _warnings_showwarning is not None:
                warnings.showwarning = _warnings_showwarning
                _warnings_showwarning = None

if not INTERACTIVE_INTERPRETER:
    try:
        colorama
    except NameError:
        from colorama import init, Fore, Back, Style
        init()


NO_STYLE = ("", "")


class ColorFormatter(Formatter):
    """
    Adds colorized formatting to log messages using :mod:`colorama` so long
    as we're not running an interactive interpreter or a debugger.
    """
    if not INTERACTIVE_INTERPRETER:
        FORMATS = {
            logging.DEBUG: (Style.DIM, Style.RESET_ALL),
            logging.WARNING: (Fore.YELLOW, Fore.RESET),
            logging.ERROR: (Fore.RED, Fore.RESET),
            logging.CRITICAL: (
                Fore.RED + Style.BRIGHT, Fore.RESET + Style.RESET_ALL)}

        # Python 2.6 uses old style classes which means we can't use
        # super().  So we construct the proper method at the class level
        # so we can safe an if statement for each function call.
        if not PY26:  # pragma: no cover
            def format(self, record):
                head, tail = self.FORMATS.get(record.levelno, NO_STYLE)
                return head + super(ColorFormatter, self).format(record) + tail
        else:  # pragma: no cover
            def format(self, record):
                head, tail = self.FORMATS.get(record.levelno, NO_STYLE)
                return head + Formatter.format(self, record) + tail

    else:
        warnings.warn_explicit(
            "Interactive interpreter or debugger is active, "
            "disabling colorized logging.", RuntimeWarning, "logger.py",
            0, module="pyfarm.core.logger")

        FORMATS = {
            logging.DEBUG: NO_STYLE,
            logging.WARNING: NO_STYLE,
            logging.ERROR: NO_STYLE,
            logging.CRITICAL: NO_STYLE}


class StandardOutputStreamHandler(logging.StreamHandler):
    """
    This is exactly the same as :class:`logging.StreamHandler` the
    only exception is we use ``sys.stdout`` by default.  This class is
    provided so it can be serialized as a string into a logging configuration
    dictionary using json.
    """
    if PY26:
        def __init__(self, stream=sys.stdout):
            logging.StreamHandler.__init__(self, stream)
    else:
        def __init__(self, stream=sys.stdout):
            super(StandardOutputStreamHandler, self).__init__(stream=stream)


class config(object):
    """
    Namespace class to store and setup the logging configuration.  You
    typically don't have to run the classmethods here manually but you may
    do so under other circumstances if you wish.
    """
    CONFIGURED = False
    DEFAULT_CONFIGURATION = {
        "version": 1,
        "root": {
            "level": os.environ.get("PYFARM_ROOT_LOGLEVEL", "DEBUG"),
            "handlers": ["stdout"],
        },
        "handlers": {
            "stdout": {
                "class": "pyfarm.core.logger.StandardOutputStreamHandler",
                "formatter": "colorized"
            }
        },
        "formatters": {
            "colorized": {
                "()": "pyfarm.core.logger.ColorFormatter",
                "datefmt": "%Y-%m-%d %H:%M:%S",
                "format":
                    "%(asctime)s %(levelname)-8s - %(name)-15s - %(message)s"
            }
        }
    }

    @classmethod
    def get(cls):
        """
        Retrieves the logging configuration.  By default this searches
        :envvar:`PYFARM_LOGGING_CONFIG` for either a json blob containing
        the logging configuration or a path to a json blob on disk.  If
        :envvar:`PYFARM_LOGGING_CONFIG` is not set then this function falls
        back on :const:`pyfarm.core.logger.DEFAULT_CONFIGURATION`.
        """
        # nothing left to do if it's not in the environment
        if "PYFARM_LOGGING_CONFIG" not in os.environ:
            return cls.DEFAULT_CONFIGURATION.copy()

        environment_config = os.environ["PYFARM_LOGGING_CONFIG"].strip()
        if not environment_config:
            raise ValueError("$PYFARM_LOGGING_CONFIG is empty")

        try:
            with open(environment_config, "r") as stream:
                try:
                    return json.load(stream)
                except ValueError:
                    raise ValueError(
                        "Failed to parse json data from %s" % stream.name)
        except (OSError, IOError):
            try:
                return json.loads(environment_config)
            except ValueError:
                raise ValueError(
                    "Failed to parse json data from $PYFARM_LOGGING_CONFIG")

    @classmethod
    def setup(cls, capture_warnings=True, reconfigure=False):
        """
        Retrieves the logging configuration using :func:`get` and
        then calls :meth:`.dictConfig` on the results.

        :type capture_warnings: bool
        :param capture_warnings:
            If True then all emissions from :mod:`warnings` should instead
            be logged

        :type reconfigure: bool
        :param reconfigure:
            If True then rerun :func:`.dictConfig` even if we've already done
            so.
        """
        if not reconfigure and cls.CONFIGURED:
            return

        dictConfig(cls.get())
        if capture_warnings:
            captureWarnings(True)

        cls.CONFIGURED = True


def getLogger(name):
    """
    Wrapper around the :func:`logging.getLogger` function which
    ensures the name is setup properly.
    """
    config.setup()
    if not name.startswith("pf."):
        name = "pf.%s" % name

    return logging.getLogger(name)


# No shebang line, this module is meant to be imported
#
# Copyright 2013 Oliver Palmer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Utilities
=========

General utility functions that are not specific to individual components
of PyFarm.
"""

from __future__ import division

import json
from functools import partial
from ast import literal_eval

try:
    from UserDict import UserDict
except ImportError:  # pragma: no cover
    from collections import UserDict

from pyfarm.core.config import read_env_bool
from pyfarm.core.enums import (
    NUMERIC_TYPES, STRING_TYPES, PY2, PY3,
    BOOLEAN_TRUE, BOOLEAN_FALSE, NONE, Values)


class ImmutableDict(dict):
    """
    A basic immutable dictionary that's built on top of Python's
    standard :class:`dict` class.  Once :meth:`__init__` has been
    run the contents of the instance can no longer be modified
    """
    def __init__(self, iterable=None, **kwargs):
        if self:
            raise RuntimeError("__init__ already run.")

        self.__writable = True
        try:
            super(ImmutableDict, self).__init__(iterable or [], **kwargs)
        except RuntimeError:  # pragma: no cover
            raise
        finally:
            del self.__writable

    # Force Python 2.x to use generators for items/keys/values
    if PY2:  # pragma: no cover
        items = dict.iteritems
        keys = dict.iterkeys
        values = dict.itervalues

    # Decorator to check if we're allowed to write
    # data to the class
    def write_required(method):  # pragma: no cover
        def wrapper(*args, **kwargs):
            if not hasattr(args[0], "__writable"):
                raise RuntimeError("Cannot modify a read-only dictionary.")
            return method(*args, **kwargs)
        return wrapper

    # Wrap the 'writable' methods using the decorator so
    # we can raise exceptions if someone tries to modify
    # the instance after __init__ is run.
    __setitem__ = write_required(dict.__setitem__)
    __delitem__ = write_required(dict.__delitem__)
    clear = write_required(dict.clear)
    pop = write_required(dict.pop)
    popitem = write_required(dict.popitem)
    setdefault = write_required(dict.setdefault)
    update = write_required(dict.update)

    # Once we've applied the decorator, we don't
    # need it anymore.
    del write_required


class PyFarmJSONEncoder(json.JSONEncoder):
    def encode(self, o):
        # Introspect dictionary objects for our
        # enum value type so we can dump out the string
        # value explicitly.  Otherwise, json.dumps will
        # dump out a tuple object instead.
        if isinstance(o, dict):
            o = o.copy()
            for key, value in o.items():
                if isinstance(value, Values):
                    o[key] = value.str

        return super(PyFarmJSONEncoder, self).encode(o)

dumps = partial(
    json.dumps,
    indent=4 if read_env_bool("PYFARM_PRETTY_JSON", False) else None,
    cls=PyFarmJSONEncoder)


class convert(object):
    """
    Namespace containing various static methods for converting data.

    Some staticmethods are named the same as builtin types. The name
    indicates the expected result but the staticmethod may not behave the
    same as the equivalently named Python object.  Read the documentation
    for each staticmethod to learn the differences, expected input and
    output.
    """
    @staticmethod
    def bytetomb(value):
        """
        Convert bytes to megabytes

        >>> convert.bytetomb(10485760)
        10.0
        """
        return value / 1024 / 1024

    @staticmethod
    def mbtogb(value):
        """
        Convert megabytes to gigabytes

        >>> convert.mbtogb(2048)
        2.0
        """
        return value / 1024

    @staticmethod
    def ston(value, types=NUMERIC_TYPES):
        """
        Converts a string to an integer or fails with a useful error
        message

        :param string value:
            The value to convert to an integer

        :raises ValueError:
            Raised if ``value`` could not be converted using
            :func:`.literval_eval`

        :raises TypeError:
            Raised if ``value`` was not converted to a float, integer, or long
        """
        # already a number
        if isinstance(value, types):
            return value

        # we only convert strings
        if not isinstance(value, STRING_TYPES):
            raise TypeError("`value` must be a string")

        value = literal_eval(value)

        # ensure we got a number out of literal_eval
        if not isinstance(value, types):
            raise ValueError("`value` did not convert to a number")

        return value

    @staticmethod
    def bool(value):
        """
        Converts ``value`` into a boolean object.  This function mainly exits
        so human-readable booleans such as 'yes' or 'y' can be handled in
        a single location.  Internally it does *not* use :func:`bool` and
        instead checks ``value`` against :const:`pyfarm.core.enums.BOOLEAN_TRUE`
        and :const:`pyfarm.core.enums.BOOLEAN_FALSE`.

        :param value:
            The value to attempt to convert to a boolean.  If this value is a
            string it will be run through ``.lower().strip()`` first.

        :raises ValueError:
            Raised if we can't convert ``value`` to a true boolean object
        """
        if isinstance(value, STRING_TYPES):
            value = value.lower().strip()

        if value in BOOLEAN_TRUE:
            return True
        elif value in BOOLEAN_FALSE:
            return False
        else:
            raise ValueError(
                "Cannot convert %r to either True or False" % value)

    @staticmethod
    def none(value):
        """
        Converts ``value`` into ``None``.  This function mainly exits
        so human-readable values such as 'None' or 'null' can be handled in
        a single location.  Internally this checks ``value``
        against :const:`pyfarm.core.enums.NONE`

        :param value:
            The value to attempt to convert to ``None``.  If this value is a
            string it will be run through ``.lower().strip()`` first.

        :raises ValueError:
            Raised if we can't convert ``value`` to ``None``
        """
        if isinstance(value, STRING_TYPES):
            value = value.lower().strip()

        if value in NONE:
            return None
        else:
            raise ValueError(
                "Cannot convert %r to None" % value)

    @staticmethod
    def list(value, sep=",", strip=True, filter_empty=True):
        """
        Converts ``value`` into a list object by splitting on ``sep``.

        :param str value:
            The string we should convert into a list

        :param str sep:
            The string that we should split ``value`` by.

        :param bool strip:
            If ``True``, strip extra whitespace from the results so
            ``'foo, bar'`` becomes ``['foo', 'bar']``

        :param bool filter_empty:
            If ``True``, any result that evaluates to ``False`` will be
            removed so ``'foo,,'`` would become ``['foo']``
        """
        if not isinstance(value, STRING_TYPES) \
                or not isinstance(sep, STRING_TYPES):
            raise TypeError("Expected a string for `value` and/or `sep`")

        # split the string
        value = value.split(sep)
        if strip:
            value = map(str.strip, value)

        # filter out empty values
        if filter_empty:
            value = filter(bool, value)

        # if we're in Python 3, we may be working with an iterable
        if not isinstance(value, list):
            value = list(value)

        return value



# No shebang line, this module is meant to be imported
#
# Copyright 2013 Oliver Palmer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Configuration Object
====================

Basic module used for reading configuration data into PyFarm
in various forms.

:const BOOLEAN_TRUE:
    set of values which will return a True boolean value from
    :func:`.read_env_bool`

:const BOOLEAN_FALSE:
    set of values which will return a False boolean value from
    :func:`.read_env_bool`
"""

import os
from ast import literal_eval
from errno import EEXIST
from functools import partial
from itertools import product
from pprint import pformat
from string import Template
from tempfile import gettempdir
from os.path import isfile, join, isdir, expanduser, expandvars, dirname

try:
    from StringIO import StringIO
except ImportError:  # pragma: no cover
    from io import StringIO

from pkg_resources import (
    DistributionNotFound, get_distribution, resource_filename)

import yaml
try:
    from yaml.loader import CLoader as Loader
except ImportError:  # pragma: no cover
    from yaml.loader import Loader

from pyfarm.core.logger import getLogger
from pyfarm.core.enums import (
    STRING_TYPES, NUMERIC_TYPES, NOTSET, LINUX, MAC, WINDOWS, range_)

logger = getLogger("core.config")

# Boolean values as strings that can match a value we
# pulled from the environment after calling .lower().
BOOLEAN_TRUE = set(["1", "t", "y", "true", "yes"])
BOOLEAN_FALSE = set(["0", "f", "n", "false", "no"])


def read_env(envvar, default=NOTSET, warn_if_unset=False, eval_literal=False,
             raise_eval_exception=True, log_result=True, desc=None):
    """
    Lookup and evaluate an environment variable.

    :param string envvar:
        The environment variable to lookup in :class:`os.environ`

    :keyword object default:
        Alternate value to return if ``envvar`` is not present.  If this
        is instead set to ``NOTSET`` then an exception will be raised if
        ``envvar`` is not found.

    :keyword bool warn_if_unset:
        If True, log a warning if the value being returned is the same
        as ``default``

    :keyword eval_literal:
        if True, run :func:`.literal_eval` on the value retrieved
        from the environment

    :keyword bool raise_eval_exception:
        If True and we failed to parse ``envvar`` with :func:`.literal_eval`
        then raise a :class:`EnvironmentKeyError`

    :keyword bool log_result:
        If True, log the query and result to INFO.  If False, only log the
        query itself to DEBUG.  This keyword mainly exists so environment
        variables such as :envvar:`PYFARM_SECRET` or
        :envvar:`PYFARM_DATABASE_URI` stay out of log files.

    :keyword string desc:
        Describes the purpose of the value being returned.  This may also
        be read in at the time the documentation is built.
    """
    if not log_result:  # pragma: no cover
        logger.debug("read_env(%s)" % repr(envvar))

    if envvar not in os.environ:
        # default not provided, raise an exception
        if default is NOTSET:
            raise EnvironmentError("$%s is not in the environment" % envvar)

        if warn_if_unset:  # pragma: no cover
            logger.warning("$%s is using a default value" % envvar)

        if log_result:  # pragma: no cover
            logger.info("read_env(%s): %s" % (repr(envvar), repr(default)))

        return default
    else:
        value = os.environ[envvar]

        if not eval_literal:
            return value

        try:
            return literal_eval(value)

        except (ValueError, SyntaxError) as e:
            if raise_eval_exception:
                raise

            args = (envvar, e)
            logger.error(
                "$%s contains a value which could not be parsed: %s" % args)
            logger.warning("returning default value for $%s" % envvar)
            return default


def read_env_bool(*args, **kwargs):
    """
    Wrapper around :func:`.read_env` which converts environment variables
    to boolean values.  Please see the documentation for
    :func:`.read_env` for additional information on exceptions and input
    arguments.

    :exception AssertionError:
        raised if a default value is not provided

    :exception TypeError:
        raised if the environment variable found was a string and could
        not be converted to a boolean.
    """
    notset = object()

    if len(args) == 1:
        kwargs.setdefault("default", notset)

    kwargs["eval_literal"] = False  # we'll handle this ourselves here
    value = read_env(*args, **kwargs)
    assert value is not notset, "default value required for `read_env_bool`"

    if isinstance(value, STRING_TYPES):
        value = value.lower()

        if value in BOOLEAN_TRUE:
            return True

        elif value in BOOLEAN_FALSE:
            return False

        else:
            raise TypeError(
                "could not convert %s to a boolean from $%s" % (
                    repr(value), args[0]))

    if not isinstance(value, bool):
        raise TypeError("expected a boolean default value for `read_env_bool`")

    return value


def read_env_number(*args, **kwargs):
    """
    Wrapper around :func:`.read_env` which will read a numerical value
    from an environment variable.  Please see the documentation for
    :func:`.read_env` for additional information on exceptions and input
    arguments.

    :exception TypeError:
        raised if we either failed to convert the value from the environment
        variable or the value was not a float, integer, or long
    """

    if len(args) == 1:
        kwargs.setdefault("default", NOTSET)

    kwargs["eval_literal"] = True
    try:
        value = read_env(*args, **kwargs)
    except ValueError:
        raise ValueError("failed to evaluate the data in $%s" % args[0])

    assert value is not NOTSET, "default value required for `read_env_number`"

    if not isinstance(value, NUMERIC_TYPES):
        raise TypeError("`read_env_number` did not return a number type object")

    return value


def read_env_strict_number(*args, **kwargs):
    """
    Strict version of :func:`.read_env_number` which will only return an integer

    :keyword number_type:
        the type of number(s) this function must return

    :exception AsssertionError:
        raised if the number_type keyword is not provided (required to check
        the type on output)

    :exception TypeError:
        raised if the type of the result is not an instance of `number_type`
    """
    number_type = kwargs.pop("number_type", None)
    assert number_type is not None, "`number_type` keyword is required for"
    value = read_env_number(*args, **kwargs)

    if not isinstance(value, number_type):
        raise TypeError("%s is not an %s object" % (repr(value), number_type))

    return value


read_env_int = partial(read_env_strict_number, number_type=int)
read_env_float = partial(read_env_strict_number, number_type=float)


class Configuration(dict):
    """
    Main object responsible for finding, loading, and
    merging configuration data.  By default this class does nothing
    until :meth:`load` is called.  Once this method is called
    :class:`Configuration` class will populate itself with data loaded
    from the configuration files.  The configuration files themselves can
    be loaded from multiple location depending on the system's setup.  For
    example on Linux you might end up attempting to load:

        * The default configuration as provided by PyFarm's source.
        * ``/etc/pyfarm/agent/agent.yml``
        * ``/etc/pyfarm/agent/1/agent.yml``
        * ``/etc/pyfarm/agent/1.2/agent.yml``
        * ``/etc/pyfarm/agent/1.2.3/agent.yml``
        * ``~/.pyfarm/agent/agent.yml``
        * ``~/.pyfarm/agent/1/agent.yml``
        * ``~/.pyfarm/agent/1.2/agent.yml``
        * ``~/.pyfarm/agent/1.2.3/agent.yml``
        * ``etc/pyfarm/agent/agent.yml``
        * ``etc/pyfarm/agent/1/agent.yml``
        * ``etc/pyfarm/agent/1.2/agent.yml``
        * ``etc/pyfarm/agent/1.2.3/agent.yml``

    :class:`Configuration` will only attempt to load data from files which
    exist on the file system when :meth:`load` is called.  If multiple files
    exist the data will be loaded from each file with the successive data
    overwriting the value from the previously loaded configuration file. So
    if you have two files containing the same data:

        * ``/etc/pyfarm/agent/agent.yml``

            .. code-block:: yaml

                env:
                    a: 0
                foo: 1
                bar: true


        * ``etc/pyfarm/agent/1.2.3/agent.yml``

            .. code-block:: yaml

                env:
                    a: 1
                    b: 1
                foo: 0

    You'll end up with a single merged configuration.  Please note that the
    only keys which will be merged in the configuration are the ``env`` key.
    Configuration files are meant to store simple data and while it can be
    used to store more complicate data it won't merge any other data
    structures.

        .. code-block:: yaml

            env:
                a: 1
                b: 1
            foo: 0
            bar: true

    :var string DEFAULT_SYSTEM_ROOT:
        The system level directory that we should look for configuration
        files in.  This path is platform dependent:

            * **Linux** - /etc/
            * **Mac** - /Library/
            * **Windows** - %ProgramData%.  An environment variable that
              varies depending on the Windows version.  See Microsoft's docs:
              https://www.microsoft.com/security/portal/mmpc/shared/variables.aspx

        The value built here will be copied onto the instance as ``system_root``

    :var string DEFAULT_USER_ROOT:
        The user level directory that we should look for configuration
        files in.  This path is platform dependent:

            * **Linux/Mac** - ~ (home directory)
            * **Windows** - %APPDATA%.  An environment variable that
              varies depending on the Windows version.  See Microsoft's docs:
              https://www.microsoft.com/security/portal/mmpc/shared/variables.aspx

        The value built here will be copied onto the instance as ``user_root``

    :var string DEFAULT_FILE_EXTENSION:
        The default file extension of the configuration files.  This will
        default to ``.yml`` and will be copied to ``file_extension`` when
        the class is instanced.

    :var string DEFAULT_LOCAL_DIRECTORY_NAME:
        A directory local to the current process which we should search
        for configuration files in.  This will default to ``etc`` and
        will be copied to ``local_dir`` when the class is instanced.

    :var string DEFAULT_PARENT_APPLICATION_NAME:
        The base name of the parent application.  This used used to build
        child directories and will default to ``pyfarm``.

    :var string DEFAULT_ENVIRONMENT_PATH_VARIABLE:
        A environment variable to search for a configuration path in.

    :var DEFAULT_TEMP_DIRECTORY_ROOT:
        The directory which will

    :param string name:
        The name of the configuration itself, typically 'master' or
        'agent'.  This may also be the name of a package such
        as 'pyfarm.agent'.  When the package name is provided
        we can usually automatically determine the version
        number.

    :param string version:
        The version the version of the program running.

    .. automethod:: _expandvars
    """
    MAX_EXPANSION_RECURSION = 10

    if LINUX:  # pragma: no cover
        DEFAULT_SYSTEM_ROOT = join(os.sep, "etc")
        DEFAULT_USER_ROOT = expanduser("~")
    elif MAC:  # pragma: no cover
        DEFAULT_SYSTEM_ROOT = join(os.sep, "Library")
        DEFAULT_USER_ROOT = expanduser("~")
    elif WINDOWS:  # pragma: no cover
        DEFAULT_SYSTEM_ROOT = expandvars("$ProgramData")
        DEFAULT_USER_ROOT = expandvars("$APPDATA")
    else:  # pragma: no cover
        logger.warning("Failed to determine default configuration roots")
        DEFAULT_SYSTEM_ROOT = None
        DEFAULT_USER_ROOT = None

    DEFAULT_FILE_EXTENSION = ".yml"
    DEFAULT_LOCAL_DIRECTORY_NAME = "etc"
    DEFAULT_PARENT_APPLICATION_NAME = "pyfarm"
    DEFAULT_ENVIRONMENT_PATH_VARIABLE = "PYFARM_CONFIG_ROOT"
    DEFAULT_TEMP_DIRECTORY_ROOT = join(
        gettempdir(), DEFAULT_PARENT_APPLICATION_NAME)

    def __init__(self, name, version=None):
        super(Configuration, self).__init__()

        self._name = name
        self.loaded = ()
        self.searched = []
        self.file_extension = self.DEFAULT_FILE_EXTENSION
        self.system_root = self.DEFAULT_SYSTEM_ROOT
        self.user_root = self.DEFAULT_USER_ROOT
        self.local_dir = self.DEFAULT_LOCAL_DIRECTORY_NAME
        self.environment_root = read_env(
            self.DEFAULT_ENVIRONMENT_PATH_VARIABLE, None)

        # If `name` is an import name and an
        # explict version was not provided then try
        # to find one automatically.
        if version is None:
            try:
                self.distribution = get_distribution(name)

            except DistributionNotFound:
                raise ValueError(
                    "%r is not a Python package so you must provide "
                    "a version." % self._name)
            else:
                self.version = self.distribution.version
                self.name = self._name.split(".")[-1]

        else:
            self.distribution = None
            self.version = version
            self.name = self._name

        self.child_dir = join(self.DEFAULT_PARENT_APPLICATION_NAME, self.name)
        self.tempdir = join(self.DEFAULT_TEMP_DIRECTORY_ROOT, self.name)

        # Create the base tempdir if it does not already
        # exist.  We're handling the exception instead of
        # using isdir() because it's possible multiple
        # processes could try to create the directory and
        # it's safer to let the file system handle it.
        try:
            os.makedirs(self.tempdir)
        except OSError as e:
            if e.errno != EEXIST:
                raise
        else:
            logger.debug("Created %r", self.tempdir)

        # Try to locate the package's built-in configuration
        # file.  This will be loaded before anything else
        # to provide the default values.
        try:
            self.package_configuration = resource_filename(
                name,
                join("etc", self._name.split(".")[-1] + self.file_extension))
        except ImportError:
            logger.warning(
                "Could not determine the default configuration file "
                "path for %s", self.name)
            self.package_configuration = None

    def split_version(self, sep="."):
        """
        Splits ``self.version`` into a tuple of individual versions.  For
        example ``1.2.3`` would be split into ``['1', '1.2', '1.2.3']``
        """
        if not self.version:
            return []

        split = self.version.split(sep)
        return [
            sep.join(split[:index]) for index, _ in enumerate(split, start=1)]

    def directories(self):
        """
        Returns a list of platform dependent directories which may contain
        configuration files.
        """
        roots = []
        versions = self.split_version()
        versions.insert(0, "")  # the 'version free' directory

        # If provided, insert the default root
        if self.system_root:  # could be empty in the environment
            roots.append(join(self.system_root, self.child_dir))

        # If provided, append the root discovered in the environment
        if self.environment_root is not None:
            roots.append(join(self.environment_root, self.child_dir))

        # If provided append the user directory
        if self.user_root:  # could be empty in the environment
            roots.append(join(self.user_root, self.child_dir))

        # If provided append a local directory
        if self.local_dir is not None:
            roots.append(join(self.local_dir, self.child_dir))

        all_directories = []
        existing_directories = []

        for root, tail in product(roots, versions):
            directory = join(root, tail)
            all_directories.append(directory)
            self.searched.append(directory)

            if isdir(directory):
                existing_directories.append(directory)

        return existing_directories

    def files(self):
        """Returns a list of configuration files."""
        directories = self.directories()
        filename = self.name + self.file_extension
        existing_files = []

        if self.package_configuration is not None:
            self.searched.insert(0, dirname(self.package_configuration))
            if isfile(self.package_configuration):
                existing_files.append(self.package_configuration)
            else:
                logger.warning(
                    "%r does not have a default configuration file. Expected "
                    "to find %r but this path does not exist.",
                    self._name, self.package_configuration)

        for directory in directories:
            filepath = join(directory, filename)

            if isfile(filepath):
                existing_files.append(filepath)

        if not existing_files:  # pragma: no cover
            logger.error(
                "No configuration file(s) %s were found in %s",
                filename, pformat(directories))

        return existing_files

    def load(self, environment=None):
        """
        Loads data from the configuration files.  Any data present
        in the ``env`` key in the configuration files will update
        ``environment``

        :param dict environment:
            A dictionary to load data in the ``env`` key from
            the configuration files into.  This would typically be
            set to ``os.environ`` so the environment itself could
            be updated.
        """
        loaded = []
        self.searched = []
        for filepath in self.files():
            with open(filepath, "rb") as stream:
                try:
                    data = yaml.load(stream, Loader=Loader)

                except yaml.YAMLError as e:  # pragma: no cover
                    logger.error("Failed to load %r: %s", filepath, e)
                    continue

            loaded.append(filepath)

            # Empty file
            if not data:
                continue

            if environment is not None and "env" in data:
                config_environment = data.pop("env")
                assert isinstance(config_environment, dict)
                environment.update(config_environment)

            elif environment is None:
                logger.warning(
                    "No environment was provided to be populated by the "
                    "configuration file(s)")

            # Update this instance with the loaded data
            self.update(data)

        # Store loaded/searched for external use
        self.loaded = tuple(loaded)
        self.searched = tuple(self.searched)

        if self.loaded:
            logger.info(
                "Loaded configuration file(s): %s", pformat(self.loaded))
        else:
            logger.warning(
                "No configuration files were loaded after searching %s",
                pformat(self.searched))

    def _expandvars(self, value):
        """
        Performs variable expansion for ``value``. This method is run when
        a string value is returned from :meth:`get` or :meth:`__getitem__`.
        The default behavior of this method is to recursively expand
        variables using sources in the following order:

            * The environment, ``os.environ``
            * The environment (from the configuration), ``env``
            * Other values in the configuration
            * ``~`` to the user's home directory

        For example, the following configuration:

        .. code-block:: yaml

            foo: foo
            bar: bar
            foobar: $foo/$bar
            path: ~/$foobar/$TEST

        Would result in the following assuming ``$TEST`` is an
        environment variable set to ``somevalue`` and the current
        user's name is ``user``:

        .. code-block:: python

            {
                "foo": "foo",
                "bar": "bar",
                "foobar": "foo/bar",
                "path": "/home/user/foo/bar/somevalue"
            }
        """
        template_values = {"temp": self.tempdir}
        template_values.update(os.environ)
        template_values.update(self.get("env", {}))
        template_values.update(**self)

        # Do recursive variable expansion until we've either
        # reached MAX_EXPANSION_RECURSION or the resulting
        # value is unchanged
        for _ in range_(self.MAX_EXPANSION_RECURSION):
            template = Template(value)
            expanded = expanduser(template.safe_substitute(**template_values))

            # Nothing left to expand
            if value == expanded:
                break
            else:
                value = expanded

        return value

    def get(self, key, default=None):
        """
        Overrides :meth:`dict.get` to provide internal variable
        expansion through :meth:`_expandvars`.
        """
        value = super(Configuration, self).get(key, default)
        if isinstance(value, STRING_TYPES):
            value = self._expandvars(value)
        return value

    def __getitem__(self, item):
        """
        Overrides :meth:`dict.__getitem__` to provide internal variable
        expansion through :meth:`_expandvars`.
        """
        value = super(Configuration, self).__getitem__(item)
        if isinstance(value, STRING_TYPES):
            value = self._expandvars(value)
        return value



# No shebang line, this module is meant to be imported
#
# Copyright 2013 Oliver Palmer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil
import tempfile
from functools import wraps
from nose.plugins.skip import SkipTest

from pyfarm.core.enums import PY26

if PY26:
    import unittest2 as unittest
else:
    import unittest


def skip_on_ci(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if "BUILDBOT_UUID" in os.environ or "TRAVIS" in os.environ:
            raise SkipTest
        return func(*args, **kwargs)
    return wrapper


def requires_ci(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if "BUILDBOT_UUID" not in os.environ or "TRAVIS" not in os.environ:
            raise SkipTest
        return func(*args, **kwargs)
    return wrapper

from pyfarm.core.enums import STRING_TYPES


def rm(path):
    try:
        os.remove(path)
    except Exception:
        pass
    try:
        shutil.rmtree(path)
    except Exception:
        pass


class TestCase(unittest.TestCase):
    TEMPDIR_PREFIX = ""
    ORIGINAL_ENVIRONMENT = {}
    temp_directories = set()

    @classmethod
    def remove(cls, path):
        assert isinstance(path, STRING_TYPES), "expected a string for `path`"

        if os.path.isfile(path):
            delete = os.remove
        elif os.path.isdir(path):
            delete = shutil.rmtree
        else:
            delete = lambda path: None

        # delete the path
        try:
            delete(path)

        except (OSError, IOError):
            pass

        else:
            if path in cls.temp_directories:
                cls.temp_directories.remove(path)

    @classmethod
    def setUpClass(cls):
        cls.ORIGINAL_ENVIRONMENT = os.environ.copy()

    @classmethod
    def mktempdir(cls):
        tempdir = tempfile.mkdtemp(prefix=cls.TEMPDIR_PREFIX)
        cls.temp_directories.add(tempdir)
        return tempdir

    def setUp(self):
        self.tempdir = self.mktempdir()
        os.environ.clear()
        os.environ.update(self.ORIGINAL_ENVIRONMENT)

    def tearDown(self):
        try:
            self.remove(self.tempdir)
            map(self.remove, self.temp_directories.copy())
        except AttributeError:
            pass

    def add_cleanup_path(self, path):
        self.addCleanup(rm, path)



# No shebang line, this module is meant to be imported
#
# Copyright 2013 Oliver Palmer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Enums
=====

Provides enum values for certain aspect of PyFarm.  See below for more
detailed information.


Operating System
----------------

Describes an operating system type.

.. csv-table:: **OperatingSystem**
    :header: Attribute, Description
    :widths: 10, 50

    LINUX, operating system on agent is a Linux variant
    WINDOWS, operating system on agent is a Windows variant
    MAC, operating system on agent is an Apple OS variant


Agent State
-----------

The last known state of the remote agent, used for making queue decisions
and locking off resources.

.. csv-table::
    :header: Attribute, Description
    :widths: 10, 50

    OFFLINE, agent cannot be reached
    ONLINE, agent is waiting for work
    DISABLED, agent is online but cannot accept work
    RUNNING, agent is currently processing work
    ALLOC, special internal state used when the agent entry is being built


Work State
----------

The state a job or task is currently in.  These values apply more directly
to tasks as job statuses are built from task status values.

.. csv-table::
    :header: Attribute, Description
    :widths: 10, 50

    PAUSED, this task cannot be assigned right now but can be once unpaused
    RUNNING, work is currently being processed
    DONE, work is finished (previous failures may be present)
    FAILED, work as failed and cannot be continued


Use Agent Address
-----------------

Describes which address should be used to contact the agent

.. csv-table::
    :header: Attribute, Description
    :widths: 10, 50

    LOCAL, use the address which was provided by the agent
    REMOTE, use the address which we received the request from
    HOSTNAME, disregard both the local IP and the remote IP and use the hostname
    PASSIVE, agent cannot be contacted but will still request work and process jobs

:const PY_MAJOR:
    the major Python version

:const PY_MINOR:
    the minor Python version

:const PY_VERSION:
    a tuple containing the major and minor Python versions

:const PY3:
    True if running Python 3

:const PY2:
    True if running Python 2

:const PY26:
    True if running Python 2.6

:const PY27:
    True if running Python 2.7

:const NOTSET:
    Instance of the object class, mainly used when None is actually
    a valid value

:const STRING_TYPES:
    A tuple of string types, provided for Python 3 backwards compatibility

:const NUMERIC_TYPES:
    A tuple of numeric types, provided for Python 3 backwards compatibility

:const INTEGER_TYPES:
    A tuple of integer types, provided for Python 3 backwards compatibility

:const BOOLEAN_TRUE:
    A set containing strings and other objects representing ``True`` under
    some conditions.  Generally used by
    :func:`pyfarm.core.utility.convert.bool`

:const BOOLEAN_FALSE:
    A set containing strings and other objects representing ``False`` under
    some conditions.  Generally used by
    :func:`pyfarm.core.utility.convert.bool`

:const NONE:
    A set containing strings and other objects which represent ``None`` under
    some conditions.  Generally used by
    :func:`pyfarm.core.utility.convert.none`

:const INTERACTIVE_INTERPRETER:
    True when we're running inside an interactive interpreter such as
    a Python shell like IPython.  This value will also be True if
    there's an active debugger.

:const OS:
    The current os type, the value will map to one of the values in
    :class:`.OperatingSystem`

:const POSIX:
    True if ``OS in (OperatingSystem.LINUX, OperatingSystem.MAC)``

:const WINDOWS:
    True if ``OS == OperatingSystem.WINDOWS``

:const LINUX:
    True if ``OS == OperatingSystem.LINUX``

:const MAC:
    True if ``OS == OperatingSystem.MAC``
"""

import sys

# Python 2.5 is not supported across the board.  If
# somehow someone made it to this point, stop here.
PY_MAJOR, PY_MINOR = sys.version_info[0:2]
PY_VERSION = (PY_MAJOR, PY_MINOR)
if PY_VERSION <= (2, 5):  # pragma: no cover
    raise RuntimeError("Python 2.5 and below is not supported")

from collections import namedtuple

NOTSET = object()

# general Python version constants which are
# used elsewhere
PY3 = PY_MAJOR == 3
PY2 = PY_MAJOR == 2
PY26 = PY_VERSION == (2, 6)
PY27 = PY_VERSION == (2, 7)

try:  # pragma: no cover
    STRING_TYPES = (str, unicode)
    NUMERIC_TYPES = (int, long, float, complex)
    INTEGER_TYPES = (int, long)
except NameError:  # pragma: no cover
    STRING_TYPES = (str, )
    NUMERIC_TYPES = (int, float, complex)
    INTEGER_TYPES = int

# constants used by pyfarm.core.utility.convert by provided
# here so they could be reused elsewhere
BOOLEAN_TRUE = set(["1", "t", "y", "true", "yes", True, 1])
BOOLEAN_FALSE = set(["0", "f", "n", "false", "no", False, 0])
NONE = set(["none", "null", "", None, 0])

try:  # pragma: no cover
    range_ = xrange
except NameError:  # pragma: no cover
    range_ = range


def Enum(classname, **kwargs):
    """
    Produce an enum object using :func:`.namedtuple`

    >>> Foo = Enum("Foo", A=1, B=2)
    >>> assert Foo.A == 1 and Foo.B == 2
    >>> FooTemplate = Enum("Foo", A=int, instance=False)
    >>> Foo = FooTemplate(A=1)
    >>> assert Foo.A == 1

    :param str classname:
        the name of the class to produce

    :keyword to_dict:
        a callable function to add to the named tuple for
        converting the internal values into a dictionary

    :keyword bool instance:
        by default calling :func:`.Enum` will produce an instanced
        :func:`.namedtuple` object, setting ``instance`` to False
        will instead produce the named tuple without instancing it
    """
    to_dict = kwargs.pop("to_dict", None)
    instance = kwargs.pop("instance", True)
    template = namedtuple(classname, kwargs.keys())

    if to_dict is not None:
        setattr(template, "to_dict", to_dict)

    return template(**kwargs) if instance else template


class Values(namedtuple("Values", ("int", "str"))):
    """
    Stores values to be used in an enum.  Each time this
    class is instanced it will ensure that the input values
    are of the correct type and unique.
    """
    # Numerical types which are specific to the enums
    # only.
    try:
        NUMERIC_TYPES = (int, long)
    except NameError:  # pragma: no cover
        NUMERIC_TYPES = (int, )

    check_uniqueness = True
    _integers = set()

    def __init__(self, *args, **kwargs):
        if not isinstance(self.int, self.NUMERIC_TYPES):
            raise TypeError("`int` must be an number")

        if not isinstance(self.str, STRING_TYPES):
            raise TypeError("`str` must be a string")

        if self.check_uniqueness and self.int in self._integers:
            raise ValueError("value %s is being reused" % self.int)
        else:
            self._integers.add(self.int)

        self._values = set([self.int, self.str])

    def __hash__(self):
        return self.str.__hash__()

    def __int__(self):
        return self.int

    def __str__(self):
        return self.str

    def __repr__(self):  # pragma: no cover
        return "%s(%s, %s)" % (
            self.__class__.__name__, self.int, repr(self.str))

    def __contains__(self, item):
        if isinstance(item, STRING_TYPES):
            return item == self.str
        elif isinstance(item, self.NUMERIC_TYPES):
            return item == self.int
        elif isinstance(item, Values):
            return item.str == self.str and item.int == self.int
        else:  # pragma: no cover
            return False

    def __eq__(self, other):
        return self.__contains__(other)

    def __gt__(self, other):
        if isinstance(other, self.NUMERIC_TYPES):
            return other < self.int
        elif isinstance(other, Values):
            return other.int < self.int
        else:
            raise NotImplementedError("Cannot compare against %s" % type(other))

    def __ge__(self, other):
        if isinstance(other, self.NUMERIC_TYPES):
            return other <= self.int
        elif isinstance(other, Values):
            return other.int <= self.int
        else:
            raise NotImplementedError("Cannot compare against %s" % type(other))

    def __lt__(self, other):
        if isinstance(other, self.NUMERIC_TYPES):
            return other > self.int
        elif isinstance(other, Values):
            return other.int > self.int
        else:
            raise NotImplementedError("Cannot compare against %s" % type(other))

    def __le__(self, other):
        if isinstance(other, self.NUMERIC_TYPES):
            return other >= self.int
        elif isinstance(other, Values):
            return other.int >= self.int
        else:
            raise NotImplementedError("Cannot compare against %s" % type(other))


def cast_enum(enum, enum_type):
    """
    Pulls the requested ``enum_type`` from ``enum`` and produce a new
    named tuple which contains only the requested data

    >>> from pyfarm.core.enums import Enum, Values
    >>> FooBase = Enum("Foo", A=Values(int=1, str="1")
    >>> Foo = cast_enum(FooBase, str)
    >>> assert Foo.A == "1"
    >>> Foo = cast_enum(FooBase, int)
    >>> assert Foo.A == 1
    >>> assert Foo._map == {"A": 1, 1: "A"}

    .. warning::
        This function does not perform any kind of caching.  For the most
        efficient usage it should only be called once per process or
        module for a given enum and enum_type combination.
    """
    enum_data = {}
    reverse_map = {}

    # construct the reverse mapping and push
    # the request type into enum_data
    for key, value in enum._asdict().items():
        reverse_map[value.int] = value.str
        reverse_map[value.str] = value.int

        if enum_type is int:
            enum_data[key] = value.int
        elif enum_type is str:
            enum_data[key] = value.str
        else:
            raise TypeError("Valid values for `enum_type` are int or str")

    class MappedEnum(
        namedtuple(
            enum.__class__.__name__, enum_data.keys())):  # pragma: no cover
        _map = reverse_map
        _enum = enum

        def __contains__(self, item):
            if item in self._map:
                return True
            else:
                for key, value in self._enum._asdict().items():
                    if item in value:
                        return True
            return False

    return MappedEnum(**enum_data)


# 1xx - work states
# NOTE: these values are directly tested test_enums.test_direct_work_values
_WorkState = Enum(
    "WorkState",
    PAUSED=Values(100, "paused"),
    RUNNING=Values(105, "running"),
    DONE=Values(106, "done"),
    FAILED=Values(107, "failed"))

# 2xx - agent states
# NOTE: these values are directly tested test_enums.test_direct_agent_values
_AgentState = Enum(
    "AgentState",
    DISABLED=Values(200, "disabled"),
    OFFLINE=Values(201, "offline"),
    ONLINE=Values(202, "online"),
    RUNNING=Values(203, "running"))

# 3xx - non-queue related modes or states
# NOTE: these values are directly tested test_enums.test_direct_os_values
_OperatingSystem = Enum(
    "OperatingSystem",
    LINUX=Values(300, "linux"),
    WINDOWS=Values(301, "windows"),
    MAC=Values(302, "mac"),
    OTHER=Values(303, "other"))

# NOTE: these values are directly tested test_enums.test_direct_agent_addr
_UseAgentAddress = Enum(
    "UseAgentAddress",
    LOCAL=Values(310, "local"),
    REMOTE=Values(311, "remote"),
    HOSTNAME=Values(312, "hostname"),
    PASSIVE=Values(313, "passive"))

# string versions of the enums above
WorkState = cast_enum(_WorkState, str)
AgentState = cast_enum(_AgentState, str)
OperatingSystem = cast_enum(_OperatingSystem, str)
UseAgentAddress = cast_enum(_UseAgentAddress, str)

# integer versions of the enums above, mainly declared for
# direct use within queries
DBWorkState = cast_enum(_WorkState, int)
DBAgentState = cast_enum(_AgentState, int)
DBOperatingSystem = cast_enum(_OperatingSystem, int)
DBUseAgentAddress = cast_enum(_UseAgentAddress, int)

RUNNING_WORK_STATES = set([
    WorkState.RUNNING])

DB_RUNNING_WORK_STATES = set([
    DBWorkState.RUNNING])

FAILED_WORK_STATES = set([
    WorkState.FAILED])

DB_FAILED_WORK_STATES = set([
    DBWorkState.FAILED])


def operating_system(plat=sys.platform):
    """
    Returns the operating system for the given platform.  Please
    note that while you can call this function directly you're more
    likely better off using values in :mod:`pyfarm.core.enums` instead.
    """
    if plat.startswith("linux"):
        return "linux"
    elif plat.startswith("win"):
        return "windows"
    elif plat.startswith("darwin"):
        return "mac"
    else:
        return "other"

INTERACTIVE_INTERPRETER = hasattr(sys, "ps1") or sys.gettrace() is not None

# operating system information
OS = operating_system()
POSIX = OS in (OperatingSystem.LINUX, OperatingSystem.MAC)
WINDOWS = OS == OperatingSystem.WINDOWS
LINUX = OS == OperatingSystem.LINUX
MAC = OS == OperatingSystem.MAC



# No shebang line, this module is meant to be imported
#
# Copyright 2013 Oliver Palmer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement

import sys
assert sys.version_info[0:2] >= (2, 6), "Python 2.6 or higher is required"

import os
from os.path import isfile
from setuptools import setup

install_requires = ["colorama", "logutils", "PyYaml"]

if "READTHEDOCS" in os.environ:
    install_requires += ["nose", "sphinx"]

if isfile("README.rst"):
    with open("README.rst", "r") as readme:
        long_description = readme.read()
else:
    long_description = ""

setup(
    name="pyfarm.core",
    version="0.8.4",
    packages=["pyfarm",
              "pyfarm.core"],
    namespace_packages=["pyfarm"],
    install_requires=install_requires,
    url="https://github.com/pyfarm/pyfarm-core",
    license="Apache v2.0",
    author="Oliver Palmer",
    author_email="development@pyfarm.net",
    description="Sub-library which contains core modules, "
                "classes, and data types which are used by other "
                "parts of PyFarm.",
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: System :: Distributed Computing"])
